! This program calculates finite-frequency surface wave sensitivity  kernels
! Some subroutines are in ffswk-sub.f90 
!
! To compile the program:
! 
!  ifort -132 -o ffswk ffswk.f90 ffswk-sub.f90
!
!
! Ying Zhou (ying.yingzhou@gmail.com)
! May 2013
!

      module constant_para
      integer, parameter :: maxsrec=200   ! total number of layers from surface
      integer, parameter :: laydim = 500  ! dimension of earth model file
      integer, parameter:: maxdat=2500    ! dimension of taper length  
      integer, parameter:: NDISP=4800    ! dimension (dispersion)   
      integer, parameter:: maxaniso = 6   ! number of kernels (ah av bh bv et rho)
      complex, parameter::   cc=(0,1)
      real, parameter :: dt = 2.0         ! maximum taper length = dt*maxdat 
      real, parameter :: pi = 3.14159265
      real, parameter :: twopi = 3.14159265*2
      real, parameter :: RR0 = 6371000.0
      real, parameter :: D2K = 6371.0*twopi / 360 ! radian to KM
      real, parameter :: R2D = 360.0 / twopi      ! radian to degree
      real, parameter :: scale_k = (10**9)*(10**8) ! convert kernel units to 1E-8 1/km^3
      integer    maxrec
      real       depth_max
      end module constant_para
 

       module eigen_functions
       use constant_para
       real  rs, av(laydim),ah(laydim),bv(laydim),bh(laydim) 
       real  rho(laydim),radius(laydim),ani(laydim)
       real*4 vphaser,gcomr,fl,wn, qinv
       real*4 ww(maxsrec),dw(maxsrec)
       real*4 uu(maxsrec),du(maxsrec)
       real*4 vv(maxsrec),dv(maxsrec)
       real    df,omega
       integer nlay,ls00,maxlay
       integer  is_love,is_uvw    
       character(len=70) dir_eigen
       end module eigen_functions
       
       module synthetic_ray
       use constant_para 
       real  scola,slong,rcola,rlong,fla,flo,dist_act
       real   xmom(6),scale
       real  tarray(maxdat)
       real  tstart,wintt
       integer maxlen
       real wkn(maxsrec,maxaniso)
       complex cz_conjg
       real    wseisbot
       end  module synthetic_ray

       program ffswk
       use constant_para
       use eigen_functions
       use synthetic_ray

       character (len=70) file_kn
       character (len=5)  stnm
       real*8 g2e(3,3),e2g(3,3)
       real*8 xn,yn,zn,r,th,ph,ts,ps,tr,pr,rad,colat_r,alon_r,xo,yo,zo
       real*8 ey1(maxdat),ey2(maxdat)
       integer l_mode

        steplon = 1.0  ! calculate kernel every 1 degree along ray 

        write(*,*)'mode l'
        read(*,*)l_mode

        write(*,*)'love(1) or rayleigh(2)'
        read(*,*)is_love
        if(is_love .eq. 1) is_uvw = 3
        if(is_love .eq. 2) is_uvw = 1

        write(*,*)'eigenfunction dir (e.g eigfs)'
        read(*,'(a)')dir_eigen

        write(*,*)'source_long source_lat depth'
        read(*,*)slong, sla, dep
        if(slong .lt. 0) slong = slong + 360

        write(*,*)'moment tensor xmom(6)'
        read(*,*)xmom
 
        write(*,*)'receiver_long receiver_lat'
        read(*,*)rlong, rla
        if(rlong .lt. 0) rlong = rlong + 360

        write(*,*)'length of measurement window (cosine taper) in seconds:'
        read(*,*)wintt

        write(*,*)'fresnel_number (3 or 2) frac(20 40)='
        read(*,*)fresnel_number,frac
        fresnel_number = fresnel_number * 2

        write(*,*)'maximum depth to record kernels'
        read(*,*)depth_max
        depth_max = depth_max * 1000


        dir_eigen = adjustl(dir_eigen)

        open(unit=70,file='kern/kernel.example')

        iunit_rev    = 601
        iunit_meca   = 602

        iunit_kern_av = 701
        iunit_kern_ah = 702
        iunit_kern_bv = 703
        iunit_kern_bh = 704
        iunit_kern_et = 705
        iunit_kern_rh = 706

        ! open files for kernel output
        open(iunit_kern_av,file="kern/example.kernel.av")
        open(iunit_kern_ah,file="kern/example.kernel.ah")
        open(iunit_kern_bv,file="kern/example.kernel.bv")
        open(iunit_kern_bh,file="kern/example.kernel.bh")
        open(iunit_kern_et,file="kern/example.kernel.et")
        open(iunit_kern_rh,file="kern/example.kernel.rh")
       

       ! use l to find omega

        call read_eigf(l_mode)

        ! convert vphase to km/s
        if(is_love .eq.1) vphase0 = vphaser/1000.0d0
        if(is_love .eq.2) vphase0 = vphaser/1000.0d0

        ! scale moment tensor to avoid numerical overflow
        scale = 1E-10                 
        do i = 1, 6
            scale = max(abs(xmom(i)),scale)
        enddo 
        do i = 1, 6
            xmom(i) = xmom(i) / scale   ! scale to avoid overflow
            xmom(i) = xmom(i) * 1.0E20
        enddo    

        ! convert latitude to co-latitude
        scola = 90.0d0 - sla
        rcola = 90.0d0 - rla

        ! source depth
        rs = RR0 - dep * 1000

        ! recalculate distance          
         cla = 90-scola
         clo = slong
         rla = 90-rcola
         rlo = rlong
         call  distaz(cla,clo,rla,rlo,gcarc,az12,az21)
         write(*,*)'recalculated epicentral distance = ', gcarc

         dist_act = gcarc

         open(iunit_rev,file="kern/example.sr")
         write(iunit_rev,*)slong,(90-scola), '15 0  4  5 S'
         write(iunit_rev,*)rlong,(90-rcola), '15 0  4  5 R'
         close(iunit_rev)

         open(unit=iunit_meca,file='kern/meca.example')
         write(iunit_meca,2)slong,(90-scola), dep, xmom, slong,(90-scola)
         close(iunit_meca)
2        format(3(f12.2),6(e12.4),"  1  ",2(f12.2))

1        format(f10.3,f10.3,e12.4,f10.2,f10.3,f10.3)

         write(*,*)'GMT output files: kern/example.sr'

        
        ! convert to radius
        ts  = scola*pi/180
        tr  = rcola*pi/180
        ps  = slong*pi/180
        pr  = rlong*pi/180

        ! find euler pole
        call euler(ts,ps,tr,pr,g2e,e2g)

        ! calculate taper length  
         maxlen = nint(wintt/dt)
         ndata = maxdat
         call costaper(maxlen, ndata, tarray)


        ! caluclate synthetic response 
        call calc_synthetic_ray()

        ! number of steps in along-ray direction
         istart_lon = 0
         max_lon_steps = (gcarc/steplon)+0.5 


        do  2000  isteplon = istart_lon,max_lon_steps

         alon_d =  (isteplon-0.5)*steplon     
         alon_r =  alon_d*pi/180

         ! calculate the size of fresnel zone 
         gamma = abs(sin(gcarc*pi/180)/ ( sin(alon_r)*sin(gcarc*pi/180-alon_r) ) )
         fresnel = sqrt(2*pi/wn/gamma)*180/pi

         ! frac: number of grid points per fresnel zone
         steplat = fresnel/frac  
         isteplat_max = frac * fresnel_number
         half = fresnel_number / 2
         
        ! cross-ray loop
        do 3000  isteplat = 1, isteplat_max

         ! from (-)*fresnel_number/2  to fresnel_number/2 
         alat_d = (-1) * half*fresnel + steplat * float(isteplat-1)
         colat_r  =  pi/2 - alat_d*pi/180

         rad = 1
         call  rtp2xyz(rad,colat_r,alon_r,xo,yo,zo)
         ! rotate to geographic coord using euler pole
         call rotate(xo,yo,zo,e2g,xn,yn,zn)
         call xyz2rtp(xn,yn,zn,r,th,ph)
         t2 = pi/2 - th
         p2 = ph 
         ! flo fla: geographic longitude and latitude         
         flo = p2*180/pi
         fla = t2*180/pi
         if(flo .lt. 0) flo = flo+360


        !!!!  kernel array  wkn()        
         call  scatter(alon_d,alat_d)

        ! output kernel
        do lyc = 1, maxrec
             lym = lyc+maxlay-maxrec
             dep_k = (RR0-radius(lym))/1000
             write(iunit_kern_av,1)flo,fla,wkn(lyc,1)*scale_k,dep_k ! alpha_v
             write(iunit_kern_ah,1)flo,fla,wkn(lyc,2)*scale_k,dep_k ! alpha_h
             write(iunit_kern_bv,1)flo,fla,wkn(lyc,3)*scale_k,dep_k ! beta_v
             write(iunit_kern_bh,1)flo,fla,wkn(lyc,4)*scale_k,dep_k ! beta_h
             write(iunit_kern_et,1)flo,fla,wkn(lyc,5)*scale_k,dep_k ! eta
             write(iunit_kern_rh,1)flo,fla,wkn(lyc,6)*scale_k,dep_k ! rho
        enddo     ! end of depth loop
              
3000     continue  ! end of cross-ray loop 
2000     continue  ! end of along ray loop      

       print *, 'kern/example.kernel.av   kern/example.kernel.ah ' 
       print *, 'kern/example.kernel.bv   kern/example.kernel.bh' 
       print *, 'kern/example.kernel.et   kern/example.kernel.rho' 
       print *, 'kernel unit is 10^-8 km^3/s' 

       close(iunit_kern_av)
       close(iunit_kern_ah)
       close(iunit_kern_bv)
       close(iunit_kern_bh)
       close(iunit_kern_et)
       close(iunit_kern_rh)

       stop
       end program ffswk


! rotate coordinate in one to another given a rotation matrix
       subroutine rotate(xo,yo,zo,rotm,xn,yn,zn)
       implicit REAL*8 (a-h,o-z)
        dimension rotm(3,3)

       xn=xo*rotm(1,1)+yo*rotm(1,2)+zo*rotm(1,3)
       yn=xo*rotm(2,1)+yo*rotm(2,2)+zo*rotm(2,3)
       zn=xo*rotm(3,1)+yo*rotm(3,2)+zo*rotm(3,3)

       return
       end
       

! subroutine to do rotation using euler angles
! input: vector (x,y,z), cosine of three euler angles 

      subroutine euler(ts,ps,tr,pr,g2e,e2g)
      IMPLICIT DOUBLEPRECISION (a-h,o-z)
      dimension g2e(3,3),e2g(3,3)

      pi2=asin(1.0d0)
      pi=2.0d0*pi2
      twopi=2.0d0*pi
      rpd=pi/180.0d0
      dpr=1.0d0/rpd

      call cart(ts,ps,xi,yi,zi)
      call cart(tr,pr,xr,yr,zr)
! pole (xk,yk,zk) = (xi,yi,zi) x (xr,yr,zr) of great circle plane between s and r
!  
      xk=yi*zr-yr*zi
      yk=xr*zi-xi*zr
      zk=xi*yr-xr*yi
      call cross(xi,yi,zi,xr,yr,zr,xk,yk,zk)
      dk=dsqrt(xk*xk+yk*yk+zk*zk)
      xk=xk/dk
      yk=yk/dk
      zk=zk/dk
! (xj,yj,zj) = (xk,yk,zk) x (xi,yi,zi)
      xj=yk*zi-yi*zk
      yj=xi*zk-xk*zi
      zj=xk*yi-xi*yk
      dj=dsqrt(xj*xj+yj*yj+zj*zj)
      xj=xj/dj
      yj=yj/dj
      zj=zj/dj

! rotation matrix cos(x',x)
      g2e(1,1)=xi
      g2e(2,1)=xj
      g2e(3,1)=xk
      g2e(1,2)=yi
      g2e(2,2)=yj
      g2e(3,2)=yk
      g2e(1,3)=zi
      g2e(2,3)=zj
      g2e(3,3)=zk

!      write(6,*)'a matrix'
!      do i=1,3
!         write(6,*)(g2e(i,j),j=1,3)
!      enddo

      e2g(1,1)=xi
      e2g(2,1)=yi
      e2g(3,1)=zi
      e2g(1,2)=xj
      e2g(2,2)=yj
      e2g(3,2)=zj
      e2g(1,3)=xk
      e2g(2,3)=yk
      e2g(3,3)=zk


      return
      end

!
!--------------------------------------------------------------------------
!
      subroutine cart(thet,phi,x,y,z)
      implicit real*8(a-h,o-z)
      s=dsin(thet)
      x=s*dcos(phi)
      y=s*dsin(phi)
      z=dcos(thet)
      return
      end
!
!-----------------------------------------------------------------------
!
      subroutine cross(sx,sy,sz,rx,ry,rz,px,py,pz)
      implicit real*8(a-h,o-z)
      px = sy*rz - sz*ry
      py = sz*rx - sx*rz
      pz = sx*ry - sy*rx
      return
      end
!
!----------------------------------------------------------------------


       
! convert a point at spherical coordinate to cartesian
!
      subroutine rtp2xyz(r,t,p,x,y,z)
      IMPLICIT DOUBLEPRECISION (a-h,o-z)

      pi2=asin(1.0d0)


      if (t.ne.pi2) go to 10
      st=1.0d0
      ct=0.0d0
      go to 20
10    st=dsin(t)
      ct=dcos(t)
20    sp=dsin(p)
      cp=dcos(p)
      x=r*st*cp
      y=r*st*sp
      z=r*ct

      return
      end
! convert a point at spherical coordinate to cartesian
!
      subroutine xyz2rtp(x,y,z,r,t,p)
      IMPLICIT DOUBLEPRECISION (a-h,o-z)

      pi2=asin(1.0d0)
      r=dsqrt(x*x+y*y+z*z)
      t=dacos(z/r)
      if (x.eq.0.0d0) then
         p=pi2
         return
      endif
      p=datan2(y,x)

      return
      end


       subroutine costaper(maxlen, ndata, tarray)
       parameter(pi=3.14159265)
       real tarray(*)
       do i =1,maxlen
       tarray(i) = 1 -  cos( 2*pi*i/maxlen)
       tarray(i) = tarray(i) / 2.0  ! normalize to max = 1.0
       enddo
       return
       end


