#!/usr/bin/perl 
#
#
# This script plots kernel mapview at a given depth
#
#  Example:   To plot SH kernel at 120 km depth

#             plot-kernel   example.kernel.bh   120
#

use Getopt::Std;

if ($#ARGV < 1) { print "\n\n Usage: plot-kernel kernel_file_name depth \n\n"; exit;}
$file_input = $ARGV[0];  $depth = $ARGV[1];

$file = "kernel.xyz";
qx{ rm -rf $file };

qx{cat $file_input | awk \'\{ if(\$4==$depth) print \$1 " " \$2 " " \$3 " " \$4  \}\' > $file};

if(! (-s $file)){ print 
"---------------------------------- \n
 Kernel not calculated at  $depth km
 Check $file_input and change plot depth and try again\n
-----------------------------------\n"; 
exit;} 

$RANGE="0/360/-90/90"; $SCALE ="Q180/6";

($xmin,$xmax,$ymin,$ymax,$zmin,$zmax) = split/\s+/,qx{minmax -C  $file};

$max = abs($zmin); if($max < $zmax) { $max = $zmax};
$max  = $max * 0.5;

if($max ==0){ print "kernel values are zero, exit \n"; exit; }

$PS = "$file.ps";
$cpt = "etc/test.cpt";

qx { make-kern-cpt.pl $max };

qx{ rm -rf $PS };

$title = "depth=$depth km";
#$label="sensitivity (x10^-8) 1/km^3";

  qx { pscoast -X1.2 -Y4  -J$SCALE -R$RANGE  -K -G255 -P  -B60/30:."$title":WSEN > $PS };
  qx { nearneighbor -fg $file -G$file.grd -I1/1 -R$RANGE  -S20   };
  qx { grdimage  $file.grd -C$cpt  -J$SCALE  -R$RANGE -K  -O -P -B>> $PS };
  qx { pscoast  -J$SCALE -R$RANGE  -K -W -P -O   -B >> $PS };
  qx { psxy kern/example.sr -H1 -J$SCALE  -R$RANGE -St0.2 -G200/0/0 -O  -K >> $PS};
  qx { psxy kern/example.sr -J$SCALE  -R$RANGE -W2 -G255/0/0 -O  -K >> $PS};
  qx { psmeca kern/meca.example -R$RANGE -J$SCALE -P -Sm0.4 -O -K >> $PS }; 
  qx { psscale -X2.5 -Y-1.2  -D0.5i/0.6i/4.0i/0.1ih -O -P   -C$cpt -I -E -B$max:"$label": >> $PS };   

  qx{ rm -rf  $file.grd };

print "output file :    $PS \n";
